package com.yaohun.questsystem.api;

import com.yaohun.questsystem.Main;
import com.yaohun.questsystem.data.PlayerData;
import com.yaohun.questsystem.data.QuestData;
import com.yaohun.questsystem.manage.DataManage;
import com.yaohun.questsystem.util.TimeType;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

public class QuestAPI {

    public static PlayerData getPlayerData(Player p){
        DataManage dataManage = Main.dataManage;
        PlayerData playerData = dataManage.getPlayerData(p);
        if(playerData == null){
            return null;
        }
        return playerData;
    }

    /*
    * 获取玩家的任务列表
    * */
    public static Set<QuestData> getPlayerQuestList(Player p){
        PlayerData data = Main.dataManage.getPlayerData(p);
        return data.getQuests();
    }

    /*
    * 获取玩家已接受任务 指定周期任务列表
    * */
    public static List<QuestData> getPlayerQuestList(Player p, TimeType timeType) {
        List<QuestData> questDataList = new ArrayList<>();
        PlayerData data = Main.dataManage.getPlayerData(p);
        for (QuestData questData : data.getQuests()){
            if(questData.getTimeType() == timeType){
                questDataList.add(questData);
            }
        }
        return questDataList;
    }

    /*
    * 获取指定任务信息数据
    * */
    public static QuestData getQuestData(String questKey){
        QuestData quest = Main.dataManage.getQuestData(questKey);
        if(quest == null){
            System.out.println("[错误 - 任务] 任务: "+questKey+" 配置文件不存在.");
            return null;
        }
        return quest;
    }

    /*
    * 增加任务进度
    * */
    public static void addQuestProgress(Player p,String questKey,int amount){
        QuestData quest = Main.dataManage.getQuestData(questKey);
        if(quest == null){
            System.out.println("[错误 - 任务] 任务: "+questKey+" 配置文件不存在.");
            return;
        }
        PlayerData data = Main.dataManage.getPlayerData(p);
        data.addQuestAmount(quest,amount);
    }

    /*
    * 设置任务进度
    * */
    public static void setQuestProgress(Player p,String questKey,int amount){
        QuestData quest = Main.dataManage.getQuestData(questKey);
        if(quest == null){
            System.out.println("[错误 - 任务] 任务: "+questKey+" 配置文件不存在.");
            return;
        }
        PlayerData data = Main.dataManage.getPlayerData(p);
        data.setQuestAmount(quest,amount);
    }

    /*
    * 强制完成任务进度
    * */
    public static void forceCompleteQuest(Player p,String questKey){
        QuestData quest = Main.dataManage.getQuestData(questKey);
        if(quest == null){
            System.out.println("[错误 - 任务] 任务: "+questKey+" 配置文件不存在.");
            return;
        }
        PlayerData data = Main.dataManage.getPlayerData(p);
        data.setQuestAmount(quest,-1);
    }
    /*
    * 强制完成任务进度并触发奖励
    * */
    public static void forceCompleteQuestFinish(Player p,String questKey,boolean finish){
        QuestData quest = Main.dataManage.getQuestData(questKey);
        if(quest == null){
            System.out.println("[错误 - 任务] 任务: "+questKey+" 配置文件不存在.");
            return;
        }
        PlayerData data = Main.dataManage.getPlayerData(p);
        data.setQuestAmount(quest,-1);
        if(finish){
            quest.sendFinishCommand(p);
        }
    }

    /*
    * 重置任务进度(若已完成也会返回0%)
    * */
    public static void resetCompleteQuest(Player p,String questKey){
        QuestData quest = Main.dataManage.getQuestData(questKey);
        if(quest == null){
            System.out.println("[错误 - 任务] 任务: "+questKey+" 配置文件不存在.");
            return;
        }
        PlayerData data = Main.dataManage.getPlayerData(p);
        data.setQuestAmount(quest,0);
    }

    /*
    * 删除任务
    * */
    public static void deletePlayerDataQuest(Player p,String questKey){
        QuestData quest = Main.dataManage.getQuestData(questKey);
        if(quest == null){
            System.out.println("[错误 - 任务] 任务: "+questKey+" 配置文件不存在.");
            return;
        }
        PlayerData data = Main.dataManage.getPlayerData(p);
        HashMap<QuestData, Integer> hashMap = data.getQuestAmount();
        hashMap.remove(quest);
        data.setQuestMap(hashMap);
    }

    /*
    * 删除所有任务
    * */
    public static void deleteAllPlayerDataQuest(Player p){
        PlayerData data = Main.dataManage.getPlayerData(p);
        HashMap<QuestData, Integer> hashMap = new HashMap<>();
        data.setQuestMap(hashMap);
    }

}
