package com.yaohun.questsystem.data;

import org.bukkit.Bukkit;

import java.util.HashMap;
import java.util.List;
import java.util.Set;

public class PlayerData {

    private String name; // 玩家名字
    private HashMap<QuestData,Integer> questAmount;

    public PlayerData(String name){
        this.name = name;
        this.questAmount = new HashMap<>();
    }

    public PlayerData(String name,HashMap<QuestData,Integer> hashMap){
        this.name = name;
        this.questAmount = hashMap;
        for (QuestData quest : hashMap.keySet()){
            System.out.println("[调试 - 输出] 任务名: "+quest.getQuestKey()+"  进度: "+getQuestAmount(quest)+"/"+quest.getNeedAmount());
        }
    }

    public String getName() {
        return name;
    }
    public HashMap<QuestData, Integer> getQuestAmount() {
        return questAmount;
    }

    public void setQuestMap(HashMap<QuestData, Integer> questMap) {
        this.questAmount = questMap;
    }

    public Set<QuestData> getQuests() {
        return this.questAmount.keySet();
    }
    // 获取玩家已完成任务数量
    public int getFinishedAmount(){
        int i = 0;
        for (QuestData quest : getQuests()){
            if(isFinished(quest)){
                i++;
            }
        }
        return i;
    }

    /*
    * 接受任务
    * */
    public void acceptQuest(QuestData quest){
        if(!isAcceptQuest(quest)){
            this.questAmount.put(quest,0);
        }
    }

    /*
    * 判断任务是否已接受
    * */
    public boolean isAcceptQuest(QuestData quest){
        if(this.questAmount.get(quest) == null){
            return false;
        }
        return true;
    }
    /*
    * 判断任务是否已完成
    * */
    public boolean isFinished(QuestData quest) {
        // 判断任务是否已接受
        if(isAcceptQuest(quest)){
            // 判断任务是否已提交
            if(this.questAmount.get(quest) <= -1){
                return true;
            }
        }
        return false;
    }
    /*
    * 提交任务
    * */
    public void submitQuest(QuestData quest) {
        // 提交任务后触发命令
        quest.sendFinishCommand(Bukkit.getPlayer(this.name));
        // 设置任务提交后的状态为 -1
        this.questAmount.put(quest, -1);
    }
    /*
    * 判断玩家是否达到要求可提交任务
    * */
    public boolean canSubmitQuest(QuestData quest) {
        // 判断玩家是否接受任务
        if (isAcceptQuest(quest)) {
            // 判断玩家是否已达到任务要求数量
            int amount = this.questAmount.get(quest);
            if(amount >= quest.getNeedAmount()){
                return true;
            }
        }
        return false;
    }

    /*
    * 获取玩家当前任务进度
    * */
    public int getQuestAmount(QuestData quest){
        if(this.questAmount.get(quest) == null){
            return 0;
        }
        return this.questAmount.get(quest);
    }

    /*增加任务进度*/
    public void addQuestAmount(QuestData quest){
        addQuestAmount(quest,1);
    }

    /*增加任务进度*/
    public void addQuestAmount(QuestData quest, int addAmount) {
        // 若玩家未接受任务直接返回
        if (!isAcceptQuest(quest)) {
            return;
        }
        // 当玩家当前任务进度为0时直接返回
        int amount = this.questAmount.getOrDefault(quest, 0);
        if (amount <= -1) {
            return;
        }
        int new_amount = amount + addAmount;
        int quest_amount = quest.getNeedAmount();
        if (new_amount > quest_amount) {
            int need_Amount = quest_amount - amount; // 得出 玩家还需要多少积分完成任务
            amount = amount + need_Amount;
            this.questAmount.put(quest, amount);
            return;
        }
        this.questAmount.put(quest, new_amount);
    }
    /*设置任务进度*/
    public void setQuestAmount(QuestData quest, int amount) {
        // 若玩家未接受任务直接返回
        if (!isAcceptQuest(quest)) {
            return;
        }
        this.questAmount.put(quest, amount);
    }

    /*
    * 获取任务的进度
    * */
    public double getProgress(QuestData quest) {
        if (!isAcceptQuest(quest)) {
            return 0.0D;
        }
        double amount = this.questAmount.get(quest);
        double d = amount / quest.getNeedAmount();
        if (d < 0.0D) {
            d = 0.0D;
        }else if (d > 1.0D) {
            d = 1.0D;
        }
        return d;
    }
    public void reload(List<QuestData> questDataList) {
        HashMap<QuestData, Integer> newData = new HashMap<>();
        for (QuestData newQuest : questDataList) {
            for (QuestData oldQuest : this.questAmount.keySet()) {
                if (newQuest.getQuestKey().equalsIgnoreCase(oldQuest.getQuestKey())) {
                    newData.put(newQuest, this.questAmount.get(oldQuest));
                }
            }
        }
        this.questAmount = newData;
    }
}
