package com.yaohun.questsystem.manage;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.yaohun.questsystem.Main;
import com.yaohun.questsystem.data.PlayerData;
import com.yaohun.questsystem.data.QuestData;
import com.yaohun.questsystem.util.TimeType;
import me.Demon.DemonPlugin.DemonAPI;
import org.bukkit.Bukkit;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;

import java.io.File;
import java.util.*;

public class DataManage {

    private HashMap<Player, PlayerData> dataHashMap = new HashMap<>();
    private HashMap<String, QuestData> questDataMap;

    private ArrayList<QuestData> questDataArrayList = new ArrayList<>();

    public DataManage(){
        File file = new File(Main.plugin.getDataFolder(),"quest.yml");
        FileConfiguration fileConfiguration = YamlConfiguration.loadConfiguration(file);
        HashMap<String, QuestData> dataHashMap = new HashMap<>();
        for (String questKey : fileConfiguration.getKeys(false)){
            QuestData questData = new QuestData(questKey,fileConfiguration);
            dataHashMap.put(questKey,questData);
            if(!questDataArrayList.contains(questData)) {
                questDataArrayList.add(questData);
            }
        }
        this.questDataMap = dataHashMap;
        DemonAPI.sendConsoleMessage("§f[§a!§f] §f任务列表 §8> §6"+questDataMap.size()+" 个");
    }

    public ArrayList<QuestData> getQuestDataArrayList() {
        return questDataArrayList;
    }

    public HashMap<String, QuestData> getQuestDataMap() {
        return questDataMap;
    }
    public QuestData getQuestData(String questKey){
        if(this.questDataMap.get(questKey) == null){
            return null;
        }
        return this.questDataMap.get(questKey);
    }

    /*
    * 载入玩家数据
    * */
    public void loadPlayerData(Player player) {
        String name = player.getName();
        String sqlJson = Main.sqlManager.getQuestData(name);
        // 创建 Gson 实例
        Gson gson = new Gson();
        // 解析 JSON 数据
        if (sqlJson != null && sqlJson.length() >= 3){
            JsonObject jsonObject = gson.fromJson(sqlJson, JsonObject.class);
            JsonArray questsArray = jsonObject.getAsJsonArray("quest");
            // 创建 HashMap 来存储结果
            HashMap<QuestData, Integer> questData = new HashMap<>();
            // 遍历 JSON 数组并将数据存储到 HashMap 中
            for (JsonElement element : questsArray) {
                JsonObject questObject = element.getAsJsonObject();
                String key = questObject.get("key").getAsString();
                QuestData quest = getQuestData(key);
                if (quest != null) {
                    int amount = questObject.get("amount").getAsInt();
                    questData.put(quest, amount);
                } else {
                    System.out.println("[错误 - 任务] " + key + " 不存在无法载入.");
                }
            }
            // 判断玩家数据库中是否已存在数据
            Main.sqlManager.createPlayerData(name);
            // 创建初始化数据
            PlayerData playerData = new PlayerData(name, questData);
            this.dataHashMap.put(player, playerData);
        }else{
            // 判断玩家数据库中是否已存在数据
            Main.sqlManager.createPlayerData(name);
            // 创建初始化数据
            PlayerData playerData = new PlayerData(name);
            this.dataHashMap.put(player, playerData);
        }
        player.sendMessage("§f[§e!§f]§7玩家 §6"+name+" §7任务数据已载入.");
    }

    /*
    * 删除玩家数据
    * */
    public void deletePlayerData(Player player){
        if(this.dataHashMap.get(player) == null){
            return;
        }
        this.dataHashMap.remove(player);
    }

    /*
    * 保存玩家数据
    * */
    public void savePlayerData(Player player){
        if(this.dataHashMap.get(player) == null){
            return;
        }
        PlayerData data = this.dataHashMap.get(player);
        JsonObject jsonObject = new JsonObject();
        JsonArray questArray = new JsonArray(); // 用于存储多个quest对象
        HashMap<QuestData,Integer> questAmount = data.getQuestAmount();
        for (QuestData quest : questAmount.keySet()){
            String questKey  = quest.getQuestKey();
            int amount = data.getQuestAmount(quest);
            JsonObject questObject = new JsonObject();
            questObject.addProperty("key", questKey);
            questObject.addProperty("amount", amount);
            questArray.add(questObject);
        }
        jsonObject.add("quest", questArray);
        String jsonString = jsonObject.toString();
        Main.sqlManager.SavePlayerData(player.getName(),jsonString);
    }

    public PlayerData getPlayerData(Player player){
        // 获取玩家是否在数据库中
        if(this.dataHashMap.get(player) == null){
            String name = player.getName();
            this.dataHashMap.put(player,new PlayerData(name));
        }
        return this.dataHashMap.get(player);
    }
    public HashMap<Player, PlayerData> getDataHashMap() {
        return dataHashMap;
    }
    public Set<Player> getQuests() {
        return this.dataHashMap.keySet();
    }
}
