package com.yaohun.questsystem.util;


import com.yaohun.questsystem.data.database.MegumiSQL;
import org.bukkit.Bukkit;

import java.sql.*;

public class SqlUtil extends MegumiSQL {
    private final String hostname;
    private final String port;
    private final String database;
    private final String username;
    private final String password;
    private Connection connection;

    public SqlUtil(String hostname, String port, String database, String username, String password) {
        this.hostname = hostname;
        this.port = port;
        this.database = database;
        this.username = username;
        this.password = password;
        this.connection = null;
    }

    public static String getMinecraftVersion() {
        String packageName = Bukkit.getServer().getClass().getPackage().getName();
        String version = packageName.substring(packageName.lastIndexOf('.') + 1);
        return version;
    }

    @Override
    public void openConnection() {
        try {
            String mcVersion = getMinecraftVersion();
            if(mcVersion.contains("1_20")){
                Class.forName("com.mysql.cj.jdbc.Driver"); // 1.20.4的mysql路径
            }else{
                Class.forName("com.mysql.jdbc.Driver"); // 1.18.2的mysql路径
            }
            this.connection = DriverManager.getConnection("jdbc:mysql://" + this.hostname + ":" + this.port + "/" + this.database + "?useSSL=false", this.username, this.password);
        } catch (SQLException e) {
            System.out.println("[日志 - 错误] 连接数据库失败!");
            this.connection = null;
        } catch (ClassNotFoundException e) {
            System.out.println("[日志 - 错误] 未找到JDBC驱动程序");
            this.connection = null;
        }
    }

    @Override
    public boolean checkConnection() {
        return this.connection != null;
    }

    @Override
    public Connection getConnection() {
        return this.connection;
    }

    @Override
    public void closeConnection() {
        if (this.connection != null) {
            try {
                this.connection.close();
                this.connection = null;
            } catch (SQLException e) {
                System.out.println("[日志 - 错误] 关闭数据库连接失败!");
                e.printStackTrace();
            }
        }
    }

    public ResultSet querySQL(String query) {
        Connection conn;
        if (checkConnection())
            conn = getConnection();
        else
            return null;
        Statement stat = null;
        try {
            stat = conn.createStatement();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        ResultSet result = null;
        try {
            if (stat != null)
                result = stat.executeQuery(query);
            else
                return null;
        } catch (SQLException e) {
            e.printStackTrace();
            return null;
        }
        return result;
    }

    public boolean updateSQL(String data) {
        Connection conn;
        if (checkConnection())
            conn = getConnection();
        else
            return false;
        Statement stat = null;
        try {
            stat = conn.createStatement();
            stat.executeUpdate(data);
            return true;
        } catch (SQLException e) {
            e.printStackTrace();
            return false;
        }
    }
}
